#!/usr/bin/perl -w



use strict;

use URI::Escape;



#Written by Jennifer Siepen June 2006

#parse a dat file to create pride XML (identification bit only)

#################################

#     Start the program         #

#################################



sub GetUniModInfo;

sub GetIdentifications;



my $result_file;

my $mgf_file;

my $correction;

my $mascot_file;

my $EXPERIMENT_TYPE="0";

my @spectrum_number;

my $ITRACKER_THRESHOLD="10";



if(!@ARGV)

{

print "WARNING: You have not entered any arguments!\n";  

print "WARNING: USAGE for MSMS= perl make_pride_xml.pl <output xml filename> <mgf file name><mascot .dat file><0>\n";

print "         USAGE for ITRAQ= perl make_pride_xml.pl <output xml filename> <mgf file name><mascot .dat file><1><correction file>\n";

exit(1);

}

elsif(scalar(@ARGV)<4)

{

print "WARNING: You have not entered enough arguments!\n";  

print "WARNING: USAGE for MSMS= perl make_pride_xml.pl <output xml filename> <mgf file name><mascot .dat file><0>\n";

print "         USAGE for ITRAQ= perl make_pride_xml.pl <output xml filename> <mgf file name><mascot .dat file><1><correction file>\n";

exit(1);

}

else

{

$result_file = $ARGV[0];

$mgf_file = $ARGV[1];

$mascot_file = $ARGV[2];

$EXPERIMENT_TYPE=$ARGV[3];



 #include iTRAQ data

 if($EXPERIMENT_TYPE == 1 && scalar(@ARGV)>4)

 {

 $correction = $ARGV[4];

 $ITRACKER_THRESHOLD = $ARGV[5];

 }

 #if there is iTRAQ data but no correction file

 elsif($EXPERIMENT_TYPE == 1)

 {

 print "WARNING: You have not entered enough arguments!\n";

 print "WARNING: USAGE for MSMS= perl make_pride_xml.pl <output xml filename> <mgf file name><mascot .dat file><0>\n"; 

 print "         USAGE for ITRAQ= perl make_pride_xml.pl <output xml filename> <mgf file name><mascot .dat  file><1><correction file>\n";

 exit(1);

 }

}

#required to get the unimod accessions

my $unimod_file = "unimod.xml";



#get the unimod modifications!

my %unimod = GetUniModInfo($unimod_file);



#get the information for this mascot file

GetIdentifications($mascot_file, $result_file);





sub GetIdentifications()

{

my $mascot_file = shift;

my $result_file = shift;

my $array_ptr = shift;



#Get the Info from the Mascot file

my $entries_counter = ParseMascotFile($mascot_file, $result_file, $mgf_file,$correction);

}



sub ParseMascotFile()

{

print "MESSAGE: Parsing Mascot file: $mascot_file...\n"; 

my $mascot_file = shift;

my $result_file = shift;

my $mgf_file = shift;

my $correction = shift;

my $array_ptr = shift;

my $params = 0;

my $summary = 0;

my $peptides = 0;

my $proteins = 0;

my $query = 0;

my %search;

my %ions;

my %results;

my %protein_group;

my %protein_details;



#Firstly get the hash of pepmass & charge - to correspond to the spectrum number

my %mgf_titles = GetMGFTitles($mgf_file);



#open the mascot file

open (MASCOTFILE,"<$mascot_file") or die "unable to open the mascot file, $mascot_file\n";

 while(my $line = <MASCOTFILE>)

 {

  if($line =~ m/name=\"parameters\"/)

  {

  $params = 1;

  }

  #end the parameters bit and start the summary

  elsif($line =~ m/name=\"summary\"/)

  {

  $params = 0;

  $summary = 1;

  }

  #end the summary bit and start the peptides

  elsif($line =~ m/name=\"peptides\"/)

  {

  $summary = 0;

  $peptides = 1;

  }

  #end the peptides bit and start the proteins

  elsif($line =~ m/name=\"proteins\"/)

  {

  $peptides = 0;

  $proteins = 1;

  }

  #end the proteins and get the query stuff

  elsif($line =~ m/name=\"query\d+\"/)

  {

  $proteins = 0;

  #get the query number

  my @tmp = split/name\=/, $line;

  $query = $tmp[1];

  $query =~ s/\D+//g;

  }

  #end the query stuff

  elsif($line =~ m/name=\"index"/)

  {

  $query = 0;

  }

  #use the query mass & charge to map id's back to MGF file

  elsif($summary>0 && $line =~ m/qexp\d+\=/)

  {

  my @tmp = split/\=/,$line;

  #the query number is tmp[0]

  $tmp[0] =~ s/qexp//;



  #the mass & intensity are tmp1

  my @get_values = split/\,/,$tmp[1];

  #round the mass to 4dp to match the mgf file

  $get_values[0] = sprintf "%.3f", $get_values[0];

  #and the charge (remove the sign)

  $get_values[1] =~ s/\+//;

  $get_values[1] =~ s/\n//;

  #now save the full name

  my $match = $get_values[0] . "-" . $get_values[1];



  $spectrum_number[$tmp[0]] = $mgf_titles{$match};

  }	  

  #get all the parameters 

  elsif($params == 1 && $line =~ m/\=/)

  {

  my @tmp = split/\=/, $line;

  $tmp[1] =~ s/\n//;

  $search{$tmp[0]} = $tmp[1];

  }

  #if it is the results summary - get the precursor information

  elsif($summary == 1 && $line =~ m/^q/)

  {

   #is it the mass

   if($line =~ m/^qmass/)

   {

   my @tmp = split/\=/, $line;

   #what is the query number?

   $tmp[0] =~ s/\D+//;

   #what is the mass

   $tmp[1] =~ s/\n//;

   $ions{'precursor_mass'}[$tmp[0]] = $tmp[1];

   }

   elsif($line =~ m/^qexp/)

   {

   my @tmp = split/\=/, $line;

   #what is the query number?

   $tmp[0] =~ s/\D+//;

   #what is the charge

   my @tmp2 = split/\,/,$tmp[1];

   $tmp2[1] =~ s/\n//;

    #is it positive?

    if($tmp2[1] =~ m/\+/)

    {

    $tmp2[1] =~ s/\D+//;

    }

    #otherwise it is negative

    else

    {

    $tmp2[1] =~ s/\D+//;

    $tmp2[1] = 1 - $tmp2[1];

    }

   $ions{'precursor_charge'}[$tmp[0]] = $tmp2[1];

   }

   elsif($line =~ m/^qintensity/)

   {

   my @tmp = split/\=/, $line;

   #what is the query number?

   $tmp[0] =~ s/\D+//;

   #what is the intensity

   $tmp[1] =~ s/\n//;

   $ions{'precursor_intensity'}[$tmp[0]] = $tmp[1];

   }

   elsif($line =~ m/^qmatch/)

   {

   my @tmp = split/\=/, $line;

   #what is the query number?

   $tmp[0] =~ s/\D+//;

   #what is the intensity

   $tmp[1] =~ s/\n//;

   $ions{'match'}[$tmp[0]] = $tmp[1];

   }

  }

  #if it is the peptide results

  elsif($peptides == 1 && $line =~ m /^q/)

  {

   #if it is the qn_pn= line

   if($line =~ m/q\d+\_p\d+\=/)

   {

   my @tmp = split/\=/, $line;

    #as long as it wasn't a '-1'

    if($tmp[1] !~ m/^\-1/)

    {

    #get the q and p numbers

    my @num = split/\_/,$tmp[0];

    $num[0] =~ s/q//;

    $num[1] =~ s/p//;

    $tmp[1] =~ s/\n//;

    $results{'result'}[$num[0]][$num[1]] = $tmp[1];

    #get the protein

    my $t = $results{'result'}[$num[0]][$num[1]];

    $t =~ s/\,\"/\"/g;	   

    my @info = split/\,/, $t; 

    #check there is a protein for this peptide

    my @tmp = split/\"/,$info[10];

    push(@{$protein_group{$tmp[1]}},$num[0] . "," . $num[1]);

    }

   }

   #otherwise if it is the substitution details

   elsif($line =~ m/q\d+\_p\d+\_subst\=/)

   {

   my @tmp = split/\=/, $line;

   my @num = split/\_/,$tmp[0];

   $num[0] =~ s/q//;

   $num[1] =~ s/p//;

   $tmp[1] =~ s/\n//;

   $results{'subst'}[$num[0]][$num[1]] = $tmp[1];

   }

  }

  #is it the protein details?

  elsif( $proteins==1 && $line =~ m/^\"/)

  {

  my @tmp = split/\=/, $line;

  #remove quotes from around the protein accession

  $tmp[0] =~ s/\"//g;

  my @tmp2 = split/\,/,$tmp[1];

  $tmp2[1] =~ s/\"//g;

  $tmp2[1] =~ s/\n//;

  #store the mass of this protein

  $protein_details{$tmp[0]}[0] = $tmp2[0];

  #and the description

  $protein_details{$tmp[0]}[1] = $tmp2[1];

  }



  #if it is the query stuff get the ions information

  elsif($query > 0)

  {

   if($line =~ m/^charge\=/)

   {

   my @tmp = split/\=/, $line;

   #is it positive?

    if($tmp[1] =~ m/\+/)

    {

    $tmp[1] =~ s/\D+//;

    }

    #otherwise it is negative

    else

    {

    $tmp[1] =~ s/\D+//;

    $tmp[1] = 1 - $tmp[1];

    }

   $ions{'charge'}[$query] = $tmp[1]; 

   }

   elsif($line =~ m/^Ions1\=/)

   {

   my @tmp = split/\=/, $line;

   $tmp[1] =~ s/\n//;

   $ions{'ions1'}[$query] = $tmp[1];

   }

   elsif($line =~ m/^Ions2\=/)

   {

   my @tmp = split/\=/, $line;

   $tmp[1] =~ s/\n//;

   $ions{'ions2'}[$query] = $tmp[1];

   }

   elsif($line =~ m/^Ions3\=/)

   {

   my @tmp = split/\=/, $line;

   $tmp[1] =~ s/\n//;

   $ions{'ions3'}[$query] = $tmp[1];

   }

  }

 }

close MASCOTFILE; 

#peptides 

my $count_peptides = UpdatePeptides(\%results,\%protein_group,\%ions,\%search,\%protein_details,$result_file,$mgf_file,$correction);

return 0;

}



sub UpdatePeptides()

{

my $ptr_hash = shift;

my $ptr_hash2 = shift;

my $ptr_hash3 = shift;

my $ptr_hash4 = shift;

my $ptr_hash5 = shift;

my $result_file = shift;

my $mgf_file = shift;

my $correction = shift;

my $ptr_array = shift;

my %results = %{$ptr_hash};

my %protein_group = %{$ptr_hash2};

my %ions = %{$ptr_hash3};

my %search = %{$ptr_hash4};

my %protein_details = %{$ptr_hash5};

#open the result_file to write too

open(RESULT, ">>$result_file") or die "unable to open the result file, $result_file\n";

my $pep_counter = 0;

my $search_ions_id = 0;

my %mgf_info;

#print "Getting the peptide information...\n";

 my $query_has_hit = 0;



#Get the MGF title information

%mgf_info = GetMGFInfo($mgf_file);



my %itracker_result;

 #Now for itracker

 if($EXPERIMENT_TYPE==1)

 {

 #get rid of spaces in the mgf name

 $mgf_file = "\"" . $mgf_file . "\"";

 $correction = "\"" . $correction . "\"";

 my $itracker = $mgf_file;

 print "MESSAGE: Running i-Tracker... \n";

 my $sys_call = "perl ./jenny_i-Tracker.pl $mgf_file $ITRACKER_THRESHOLD $correction";

 system($sys_call);

 print "MESSAGE: Finished Running Itracker\n ...now formatting the output $mgf_file\n";

 #what is the name of the itracker file

 $itracker =~ s/\.mgf\"$/itq\.mcn\"/; 

 #now get the itracker info

 %itracker_result = GetItrackerResult($itracker);

 }

print "MESSAGE: Merging identifications and quantitations...\n" if $EXPERIMENT_TYPE==1;



#for all the queries

#for all the proteins

my $count = 0;

 #here we deal with each of the proteins!

 foreach my $protein (keys %protein_group)

 {

 $count++;

 my $progress = $count / keys(%protein_group) * 100;

 print "PROGRESS: $progress\n";



 #Does it have valid peptides for the id?

 my $valid_id = 0;



  #for all the peptides

  for(my $pep=0 ; $pep<scalar(@{$protein_group{$protein}}) ; $pep++)

  {

  my @tmp = split/\,/,$protein_group{$protein}[$pep];

  my $q = $tmp[0];

  my $p = $tmp[1];

  #for the top 3 ranking peptides

   if($p<4)

   {

    #is there actually an asscoiated result?

    if($results{'result'}[$q][$p] && $results{'result'}[$q][$p] !~ /^\-1/)

    {

    $valid_id = 1;

    last;

    }

   }

  }

  #if this protein id is nnot supported by any good pep ids move on to the next

  if($valid_id == 0)

  {

  next;

  }  



 print RESULT "\t\t<GelFreeIdentification>\n";

 print RESULT "\t\t\t<Accession>$protein</Accession>\n";

 print RESULT "\t\t\t<Database>$search{'DB'}</Database>\n";



  for(my $pep=0 ; $pep<scalar(@{$protein_group{$protein}}) ; $pep++)

  { 

  my @tmp = split/\,/,$protein_group{$protein}[$pep];

  my $q = $tmp[0];

  my $p = $tmp[1];



  #This value tells us wether this query had any peps go into the peptide table

  $query_has_hit = 0;

  $search_ions_id = 0;



   #forall the peptides/rank

   if($p<4)

   {

    #split the line

    if($results{'result'}[$q][$p] && $results{'result'}[$q][$p] !~ /^\-1/)

    {

    #bug fix, remove any comma from protein name 

    $results{'result'}[$q][$p] =~ s/\,\"/\"/g;	   

    my @info = split/\,/, $results{'result'}[$q][$p]; 



    #check there is a protein for this peptide

    my @tmp = split/\"/,$info[10];



    #get the start and end residues (elements 2 and 3)

    my @protein_info = split/\:/,$info[10];



    #missed cleavages

    my $misscleave = $info[0];

    my $mrcalc = $info[1];

    my $mrexpt = $info[1] + $info[2];

    my $sequence = $info[4];

    my $score = $info[7];

    my $ionsmatched = $info[3];



    #increment query_has_hit to show we have updated results for this pep

    $query_has_hit++; 

    $pep_counter++;

    my $protein_id = $tmp[1];



    #Now write the xml for this peptide

    print RESULT "\t\t\t<PeptideItem>\n";

    print RESULT "\t\t\t\t<Sequence>$sequence</Sequence>\n";

    print RESULT "\t\t\t\t<Start>$protein_info[2]</Start>\n";

    print RESULT "\t\t\t\t<End>$protein_info[3]</End>\n";

    print RESULT "\t\t\t\t<SpectrumReference>$spectrum_number[$q]</SpectrumReference>\n"; 

    #Need to add modifications for this peptide, variable and fixed seperately

    #firstly the variable

    #split the var mods string

    my @varmod = split//, $info[6];

     #for all the var mods

     for(my $v=0 ; $v<scalar(@varmod) ; $v++)

     {

      #if there is a var mod in this position

      if($varmod[$v] >0)

      {

      #info is in search hash, what delta value is it?

      my $delta = "delta" . $varmod[$v];



      #get the var mod from the file

      my @mod = split/\,/,$search{$delta};



      my $mod_id=0;

      #remove any rubbish (such as letters in brackets)

      $mod[1] =~ s/\([A-Z,a-z]+\)//;

      #remove any whitespace from the end

      $mod[1] =~ s/\s$//g;

      #make the mod lowercase HERE

      $mod[1] =~ tr/[A-Z]/[a-z]/;



       #and can we get a unimod accession for this      

       if($unimod{$mod[1]}[0])

       {

       print RESULT "\t\t\t\t<ModificationItem>\n";

       print RESULT "\t\t\t\t\t<ModLocation>$v</ModLocation>\n";

       print RESULT "\t\t\t\t\t<ModAccession>$unimod{$mod[1]}[0]</ModAccession>\n";

       print RESULT "\t\t\t\t\t<ModDatabase>UNIMOD</ModDatabase>\n";

       print RESULT "\t\t\t\t\t<ModMonoDelta>$unimod{$mod[1]}[2]</ModMonoDelta>\n";

       print RESULT "\t\t\t\t\t<ModAvgDelta>$unimod{$mod[1]}[1]</ModAvgDelta>\n";

       print RESULT "\t\t\t\t</ModificationItem>\n"; 

       }

       else

       {

       #we don't have a unimod accession for this mod

       print RESULT "\t\t\t\t<ModificationItem>\n";
       print RESULT "\t\t\t\t\t<ModLocation>$v</ModLocation>\n";
       print RESULT "\t\t\t\t\t<ModAccession>$mod[1]</ModAccession>\n";
       print RESULT "\t\t\t\t\t<ModDatabase>UNKNOWN</ModDatabase>\n";
       print RESULT "\t\t\t\t</ModificationItem>\n";

       }

      }

     }     



     # are there any Fixed Mods?

     my @fix_mods;

     if($search{'MODS'})

     {

     @fix_mods = split/\,/,$search{'MODS'};

      #for each of the mods

      for(my $i=0 ; $i<scalar(@fix_mods) ; $i++)

      {

      my @residues = GetFixedMods($fix_mods[$i]);

       #for all the residues

       for(my $j=0 ; $j<scalar(@residues) ; $j++)

       {

       #Is the residue a terminus?

        if($residues[$j] =~ m/term/)

        {

        my @tmp = split/\:/,$residues[$j];

         #if it is X then all terminus 

	 if($tmp[1] eq "X" )

	 {

	 $residues[$j] = "\\S+";

	 }

	 else

	 {

   	 $residues[$j] = "^" . $tmp[1];

	 }

        }

        #do any match the sequence?

        if($sequence =~ m/$residues[$j]/)

        {

        #if they do print get unimod stuff and print cvparam

        #get the position of the matching residue

        my $position = length($`)+1;

        #remove any rubbish (such as letters in brackets)

        $fix_mods[$i] =~ s/\([A-Z,a-z]+\)//;

        #remove any whitespace from the end

        $fix_mods[$i] =~ s/\s$//g;

        #make the mod lowercase HERE

        $fix_mods[$i] =~ tr/[A-Z]/[a-z]/;

         #and can we get a unimod accession for this      

         if($unimod{$fix_mods[$i]})

         {

         print RESULT "\t\t\t\t<ModificationItem>\n";

         print RESULT "\t\t\t\t\t<ModLocation>$position</ModLocation>\n";

         print RESULT "\t\t\t\t\t<ModAccession>$unimod{$fix_mods[$i]}[0]</ModAccession>\n";

         print RESULT "\t\t\t\t\t<ModDatabase>UNIMOD</ModDatabase>\n";

         print RESULT "\t\t\t\t\t<ModMonoDelta>$unimod{$fix_mods[$i]}[2]</ModMonoDelta>\n";

         print RESULT "\t\t\t\t\t<ModAvgDelta>$unimod{$fix_mods[$i]}[1]</ModAvgDelta>\n";

	 print RESULT "\t\t\t\t</ModificationItem>\n"; 

         }

         else

         {

	 #we don't have a unimod accession for this mod
	print RESULT "\t\t\t\t<ModificationItem>\n";
	print RESULT "\t\t\t\t\t<ModLocation>$position</ModLocation>\n";
	print RESULT "\t\t\t\t\t<ModAccession>$fix_mods[$i]</ModAccession>\n";
	print RESULT "\t\t\t\t\t<ModDatabase>UNKNOWN</ModDatabase>\n";
	print RESULT "\t\t\t\t</ModificationItem>\n";

         }

        }

       }

      }

     }



    #These are the peptideItem additional parameters

    print RESULT "\t\t\t\t<additional>\n";



    #The Mascot Peptide score

    print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000069\" name=\"Mascot score\" value=\"$score\" />\n";



    #The search engine settings

     #if there any fixed mods

     if($search{'MODS'})

     {

     print RESULT "\t\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000072\" name=\"Fixed modification setting\" value=\"$search{'MODS'}\" />\n";

     }

     if($search{'IT_MODS'})

     {

     print RESULT "\t\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000073\" name=\"Variable modification setting\" value=\"$search{'IT_MODS'}\" />\n";

     }

    print RESULT "\t\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000074\" name=\"Maximum Missed Cleavages Setting\" value=\"$search{'PFA'}\" />\n";

    #and end the search engine settings

    #is it monoisotopic?

     if($search{'MASS'} =~ /Monoisotopic/)

     {

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000076\" name=\"Mass value type setting monoisotopic\"/>\n";

     }

     else

     {

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000077\" name=\"Mass value type setting average\"/>\n";

     }

    print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000078\" name=\"Peptide mass tolerance setting\" value=\"$search{'TOL'}\" />\n";

     #what are the units?

     if($search{'TOLU'} =~ /Da/)

     {

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000086\" name=\"mass error type setting Daltons\" />\n";

     }

     elsif($search{'TOLU'} =~ /ppm/)

     {

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000083\" name=\"mass error type setting ppm\" />\n";

     }

     elsif($search{'TOLU'} =~ /mmu/)

     {

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000084\" name=\"mass error type setting mmu\" />\n";

     }

     elsif($search{'TOLU'} =~ /\%/)

     {

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000085\" name=\"mass error type setting percent\" />\n";

     }

     else

     {

     print "WARNING: Unrecognised units for peptide error\n";

     }

    print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000091\" name=\"Rank\" value=\"$p\" />\n";

    print RESULT "\t\t\t\t\t<userParam name=\"mrcalc\" value=\"$mrcalc\" />\n";

    print RESULT "\t\t\t\t\t<userParam name=\"mrexpt\" value=\"$mrexpt\" />\n";

    print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000055\" name=\"Ions\" value=\"$ionsmatched\" />\n";



     if($EXPERIMENT_TYPE == 1)

     {

print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000118\" value=\"$itracker_result{'area1'}[$spectrum_number[$q]]\" name=\"iTRAQ intensity 114\"/>\n";

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000119\" value=\"$itracker_result{'area2'}[$spectrum_number[$q]]\" name=\"iTRAQ intensity 115\"/>\n";

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000120\" value=\"$itracker_result{'area3'}[$spectrum_number[$q]]\" name=\"iTRAQ intensity 116\"/>\n";

     print RESULT "\t\t\t\t\t<cvParam cvLabel=\"PRIDE\" accession=\"PRIDE:0000121\" value=\"$itracker_result{'area4'}[$spectrum_number[$q]]\" name=\"iTRAQ intensity 117\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'114_114'}[$spectrum_number[$q]]\" name=\"114_114\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'114_115'}[$spectrum_number[$q]]\" name=\"114_115\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'114_116'}[$spectrum_number[$q]]\" name=\"114_116\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'114_117'}[$spectrum_number[$q]]\" name=\"114_117\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'115_114'}[$spectrum_number[$q]]\" name=\"115_114\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'115_115'}[$spectrum_number[$q]]\" name=\"115_115\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'115_116'}[$spectrum_number[$q]]\" name=\"115_116\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'115_117'}[$spectrum_number[$q]]\" name=\"115_117\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'116_114'}[$spectrum_number[$q]]\" name=\"116_114\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'116_115'}[$spectrum_number[$q]]\" name=\"116_115\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'116_116'}[$spectrum_number[$q]]\" name=\"116_116\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'116_117'}[$spectrum_number[$q]]\" name=\"116_117\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'117_114'}[$spectrum_number[$q]]\" name=\"117_114\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'117_115'}[$spectrum_number[$q]]\" name=\"117_115\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'117_116'}[$spectrum_number[$q]]\" name=\"117_116\"/>\n";

     print RESULT "\t\t\t\t\t<userParam value=\"$itracker_result{'117_117'}[$spectrum_number[$q]]\" name=\"117_117\"/>\n";



     print RESULT "\t\t\t\t\t<userParam name=\"intensity capture method\" value=\"iTracker\"/>\n";

     }

    print RESULT "\t\t\t\t</additional>\n";

    print RESULT "\t\t\t</PeptideItem>\n";

    }

   }

  }  

 print RESULT "\t\t\t<SearchEngine>Mascot</SearchEngine>\n"; 

 print RESULT "\t\t</GelFreeIdentification>\n";

 }

close RESULT;

return $pep_counter;

}



sub GetFixedMods()

{

my $mod = shift;

my @residues;

my $found = 0; 

$mod =~ s/\(/\\\(/;

$mod =~ s/\)/\\\)/;



#open the mascot mods file

open(FILE,"<mod_file") or print "WARNING: unable to open the mascot mod_file, this is likely to mean no fixed mods are reported in the final result\n";

 while(my $line = <FILE>)

 {

  if($line=~ m/^Title\:$mod/)

  {

  $found=1;

  }

  elsif($line=~ m/^\*/)

  {

  $found = 0;

  }

  elsif($line=~ m/^Residues\:/ && $found ==1)

  {

  $line =~ s/Residues\://;

  my @tmp = split/\s+/,$line;



  push(@residues,$tmp[0]);



  }



  elsif($line=~ m/^Nterm/ && $found ==1)



  {



  push(@residues,"Nterm:X");



  }



  elsif($line=~ m/^Cterm/ && $found ==1)



  {



  push(@residues,"Cterm:X");



  }



  elsif($line=~ m/^ResiduesCterm/ && $found ==1)



  {



  $line =~ s/ResiduesCterm\://;



  my @tmp = split/\s+/,$line;



  $tmp[0] = "Cterm:" . $tmp[0];



  push(@residues,$tmp[0]);



  }



  elsif($line=~ m/^ResiduesNterm/ && $found ==1)



  {



  $line =~ s/ResiduesNterm\://;



  my @tmp = split/\s+/,$line;



  $tmp[0] = "Nterm:" . $tmp[0];



  push(@residues,$tmp[0]);



  }



 }



close FILE;











return @residues;







}







sub GetUniModInfo()



{







my $file = shift;



my %unimod;







#now need to get the corresponding mod name from the unimod xml



open(UNIMOD,"<unimod.xml") or die "unable to open the unimod file, unimod.xml\n";



 while(my $uniline = <UNIMOD>)



 {



  #only if it is a modification entry



  if($uniline =~ m/^\s+\<modifications_row/)



  {



   $uniline =~ s/^\s+//;



   my $record = 0;



   my $avge = 0;



   my $mono = 0;



   



   #get the info



   my @tmp = split/[\=\"]/,$uniline;



    



    #for each of these get the name values



    for(my $i=1 ; $i<scalar(@tmp) ; $i++)



    {



    $tmp[$i] =~ s/^\s+//;



        



     #get the record id



     if($tmp[$i] eq "record_id")



     {



     $record = $tmp[$i+2]; 



     }



     elsif($tmp[$i] eq "full_name")



     {



     #lowercase



     $tmp[$i+2] =~ tr/[A-Z]/[a-z]/;



     $unimod{$tmp[$i+2]}[0] = $record;



     $unimod{$tmp[$i+2]}[1] = $avge;



     $unimod{$tmp[$i+2]}[2] = $mono;



     }



     elsif($tmp[$i] eq "code_name")



     {



     #lowercase



     $tmp[$i+2] =~ tr/[A-Z]/[a-z]/;



     $unimod{$tmp[$i+2]}[0] = $record;



     $unimod{$tmp[$i+2]}[1] = $avge;



     $unimod{$tmp[$i+2]}[2] = $mono;



     }



     elsif($tmp[$i] eq "avge_mass")



     {



     $avge = $tmp[$i+2];



     }



     elsif($tmp[$i] eq "mono_mass")



     {



     $mono = $tmp[$i+2];



     }



    }       



  } 



 } 



 



close UNIMOD;







return %unimod;



}







sub GetItrackerResult()



{



my $file = shift;



my %results;



my $count = 0;







#remove the " from the filename



$file =~ s/\"//g;







#open the itracker file



open(FILE,"<$file") or die "unable to open the itracker results, *$file*\n";



 while(chomp(my $line = <FILE>))



 {



 $count++;



 if($count>2 && $line=~m/^\S+/)



  {



  #split the line



  #must account for the fact there could be commas in the title



  my @tmp = split/\,/,$line;



  #how many commas are there?



  my $size = scalar(@tmp);



  #the areas



  $results{'area1'}[$count-2] = $tmp[$size-20];



  $results{'area2'}[$count-2] = $tmp[$size-19];



  $results{'area3'}[$count-2] = $tmp[$size-18];



  $results{'area4'}[$count-2] = $tmp[$size-17];



  #the ratios



  $results{'114_114'}[$count-2] = $tmp[$size-16];



  $results{'115_114'}[$count-2] = $tmp[$size-15];



  $results{'116_114'}[$count-2] = $tmp[$size-14];



  $results{'117_114'}[$count-2] = $tmp[$size-13];



  $results{'114_115'}[$count-2] = $tmp[$size-12];



  $results{'115_115'}[$count-2] = $tmp[$size-11];



  $results{'116_115'}[$count-2] = $tmp[$size-10];



  $results{'117_115'}[$count-2] = $tmp[$size-9];



  $results{'114_116'}[$count-2] = $tmp[$size-8];



  $results{'115_116'}[$count-2] = $tmp[$size-7];



  $results{'116_116'}[$count-2] = $tmp[$size-6];



  $results{'117_116'}[$count-2] = $tmp[$size-5];



  $results{'114_117'}[$count-2] = $tmp[$size-4];



  $results{'115_117'}[$count-2] = $tmp[$size-3];



  $results{'116_117'}[$count-2] = $tmp[$size-2];



  $results{'117_117'}[$count-2] = $tmp[$size-1];



  }  



 }



close FILE; 







print "MESSAGE: finished getting the itracker results\n";







return %results;



}







sub GetMGFInfo()



{







my $mgf_file = shift;



my %mgf_info;



my $count=0;



my @a114;



my @a115;



my @a116;



my @a117;







 #open the mgf



 open(FILE,"<$mgf_file") or die "unable to open the mgf file, $mgf_file\n";



  while(my $line = <FILE>)



  { 



  



  $line =~s/[\r\n]$//g; 

my $name;

  



   #if the line starts with a BEGIN IONS increment count



   if($line =~ m/^BEGIN IONS/)



   {



   $count++;



   @a114=();



   @a115=();



   @a116=();



   @a117=();



   }



   elsif($line =~ m/^PEPMASS/)



   {



   $line =~ s/[\r\n]$//g;



   $line =~ s/PEPMASS=//;

   

   #make sure there is no intensity after the mass

   $line =~ s/^\s+//;

   my @tmp = split/\s+/,$line;

   $line = $tmp[0];



   #limit the pepmass to 4dp

   $line = sprintf "%.3f", $line;

   $name = $line;

   #$mgf_info{$line}[0] = $count;



   }

   elsif($line=~ m/^CHARGE/)

   {

   $line =~ s/[\r\n]$//g;

   $line =~ s/CHARGE=//;

   $name.= "_" . $line;

   $mgf_info{$name}[0] = $count; 

   }

   elsif($line =~ m/^114/)

   {



   my @tmp = split/\s+/,$line;



   push(@a114,$tmp[1]);



   }



   elsif($line =~ m/^115/)



   {



   my @tmp = split/\s+/,$line;



   push(@a115,$tmp[1]);



   }



   elsif($line =~ m/^116/)



   {



   my @tmp = split/\s+/,$line;



   push(@a116,$tmp[1]);



   }



   elsif($line =~ m/^117/)



   {



   my @tmp = split/\s+/,$line;



   push(@a117,$tmp[1]);



   }



   elsif($line =~ m/^END IONS/)

   {

    $name ="";

    $mgf_info{'114'}[$count] = 0;



    #get the maximum 114 ion



    for(my $i=0 ; $i<scalar(@a114) ; $i++)



    {



     if($a114[$i]> $mgf_info{'114'}[$count])



     {



     $mgf_info{'114'}[$count] = $a114[$i];



     }  



    }



    



    $mgf_info{'115'}[$count] = 0;



    #get the maximum 115 ion



    for(my $i=0 ; $i<scalar(@a115) ; $i++)



    {



     if($a115[$i]> $mgf_info{'115'}[$count])



     {



     $mgf_info{'115'}[$count] = $a115[$i];



     }  



    }



    



    $mgf_info{'116'}[$count] = 0;



    #get the maximum 116 ion



    for(my $i=0 ; $i<scalar(@a116) ; $i++)



    {



     if($a116[$i]> $mgf_info{'116'}[$count])



     {



     $mgf_info{'116'}[$count] = $a116[$i];



     }  



    }



    



    $mgf_info{'117'}[$count] = 0;



    #get the maximum 117 ion



    for(my $i=0 ; $i<scalar(@a117) ; $i++)



    {



     if($a117[$i]> $mgf_info{'117'}[$count])



     {



     $mgf_info{'117'}[$count] = $a117[$i];



     }  



    }



   }



  }



 close FILE;







return %mgf_info;







}











sub GetMGFTitles ()



{







my $file = shift;



my %titles;



my $count=1;







#get all the title



my $name="";

my $mass;

my $charge;

open(MGF,"<$mgf_file") or die "unable to open the MGF file, $file\n";



 while(my $line = <MGF>)



 {



  if($line =~ m/^PEPMASS/)



  {



  my @tmp = split/\=/,$line;



  $tmp[1] =~ s/\n//;

  #round to 4dp to ease matching with mascot!  

  my @tmp2 = split/\s+/,$tmp[1];

  $tmp[1] = $tmp2[0];

  $tmp[1] = sprintf "%.3f", $tmp[1];

  $mass = $tmp[1];

  }

  elsif($line =~ m/^CHARGE/)

  {

  my @tmp = split/\=/,$line;



  $tmp[1] =~ s/\n//;

  #remove the sign

  $tmp[1] =~ s/\+//;

  $charge = $tmp[1];

  }	  

  elsif($line =~ m/^END\s+IONS/) 

  {

  my $match = $mass . "-" . $charge;

  $titles{$match} = $count;  

  $mass = "";

  $charge = ""; 

  $count++;

  }  

 }

close MGF; 



return %titles;



}	



